import uuid

from django.db import models
from edc_model.models import BaseUuidModel

from ..model_mixins import UpdatesOrCreatesRegistrationModelMixin


class SubjectModelOne(UpdatesOrCreatesRegistrationModelMixin, BaseUuidModel):
    screening_identifier = models.CharField(max_length=25, null=True)

    registration_identifier = models.UUIDField(unique=True, default=uuid.uuid4)

    dob = models.DateField(null=True)

    @property
    def registration_unique_field(self):
        return "registration_identifier"


class SubjectModelTwo(UpdatesOrCreatesRegistrationModelMixin, BaseUuidModel):

    """Note: registration_unique_field is overridden."""

    subject_identifier = models.CharField(max_length=25, null=True)

    dob = models.DateField(null=True)

    @property
    def registration_unique_field(self):
        return "subject_identifier"


class SubjectModelThree(UpdatesOrCreatesRegistrationModelMixin, BaseUuidModel):

    """Note: registration_unique_field is overridden."""

    subject_identifier = models.CharField(max_length=25, null=True)

    my_identifier = models.UUIDField(default=uuid.uuid4)

    dob = models.DateField(null=True)

    @property
    def registration_unique_field(self):
        return "my_identifier"

    @property
    def registered_model_unique_field(self):
        return "registration_identifier"



from dateutil.relativedelta import relativedelta
from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase
from django.test.utils import override_settings
from edc_sites.tests import SiteTestCaseMixin
from edc_sites.utils import add_or_update_django_sites
from edc_utils import get_utcnow
from multisite import SiteID

from edc_registration.exceptions import RegisteredSubjectError
from edc_registration.models import RegisteredSubject

from .models import SubjectModelOne, SubjectModelThree, SubjectModelTwo


class TestRegistration(SiteTestCaseMixin, TestCase):
    def test_creates_registered_subject(self):
        obj = SubjectModelOne.objects.create(screening_identifier="12345")
        try:
            RegisteredSubject.objects.get(
                registration_identifier=obj.to_string(obj.registration_identifier)
            )
        except ObjectDoesNotExist:
            self.fail("RegisteredSubject was unexpectedly not created")

    def test_updates_registered_subject(self):
        SubjectModelOne.objects.create(
            screening_identifier="12345", dob=get_utcnow() - relativedelta(years=5)
        )

        new_dob = get_utcnow().date()
        obj = SubjectModelOne.objects.get(screening_identifier="12345")
        obj.dob = new_dob
        obj.save()

        rs = RegisteredSubject.objects.get(
            registration_identifier=obj.to_string(obj.registration_identifier)
        )
        self.assertEqual(rs.dob, new_dob)

    def test_creates_registered_subject_overridden(self):
        """Assert creates RegisteredSubject with registration_unique_field overridden."""
        obj = SubjectModelTwo.objects.create(subject_identifier="12345")
        try:
            RegisteredSubject.objects.get(subject_identifier=obj.subject_identifier)
        except ObjectDoesNotExist:
            self.fail("RegisteredSubject was unexpectedly not created")

    def test_updates_registered_subject_overridden(self):
        """Assert updates RegisteredSubject with registration_unique_field overridden."""
        SubjectModelTwo.objects.create(
            subject_identifier="12345", dob=get_utcnow() - relativedelta(years=5)
        )

        new_dob = get_utcnow().date()
        obj = SubjectModelTwo.objects.get(subject_identifier="12345")
        obj.dob = new_dob
        obj.save()

        rs = RegisteredSubject.objects.get(subject_identifier=obj.subject_identifier)
        self.assertEqual(rs.dob, new_dob)

    def test_creates_registered_subject_overridden2(self):
        """Assert creates RegisteredSubject with registration_unique_field overridden."""
        obj = SubjectModelThree.objects.create(subject_identifier="12345")
        try:
            RegisteredSubject.objects.get(subject_identifier=obj.subject_identifier)
        except ObjectDoesNotExist:
            self.fail("RegisteredSubject was unexpectedly not created")

    def test_updates_registered_subject_overridden2(self):
        """Assert updates RegisteredSubject with registration_unique_field overridden."""
        SubjectModelThree.objects.create(
            subject_identifier="12345", dob=get_utcnow() - relativedelta(years=5)
        )

        new_dob = get_utcnow().date()
        obj = SubjectModelThree.objects.get(subject_identifier="12345")
        obj.dob = new_dob
        obj.save()

        rs = RegisteredSubject.objects.get(subject_identifier=obj.subject_identifier)
        self.assertEqual(rs.dob, new_dob)

    def test_subject_identifier_as_uuid(self):
        obj = SubjectModelOne.objects.create(screening_identifier="12345")
        rs = RegisteredSubject.objects.get(
            registration_identifier=obj.to_string(obj.registration_identifier)
        )
        self.assertFalse(rs.subject_identifier_is_set)

    def test_masks_if_not_set(self):
        obj = SubjectModelOne.objects.create(screening_identifier="12345")
        rs = RegisteredSubject.objects.get(
            registration_identifier=obj.to_string(obj.registration_identifier)
        )
        self.assertEqual(str(rs), "<identifier not set>")
        rs.subject_identifier = "ABCDEF"
        rs.save()
        rs = RegisteredSubject.objects.get(
            registration_identifier=obj.to_string(obj.registration_identifier)
        )
        self.assertEqual(str(rs), "ABCDEF")

    def test_cannot_change_subject_identifier(self):
        obj = SubjectModelOne.objects.create(screening_identifier="12345")
        rs = RegisteredSubject.objects.get(
            registration_identifier=obj.to_string(obj.registration_identifier)
        )
        rs.subject_identifier = "ABCDEF"
        rs.save()
        rs.subject_identifier = "WXYZ"
        self.assertRaises(RegisteredSubjectError, rs.save)

    @override_settings(SITE_ID=SiteID(10))
    def test_site1(self):
        add_or_update_django_sites(single_sites=self.default_sites, verbose=False)
        obj = SubjectModelOne.objects.create(screening_identifier="12345")
        rs = RegisteredSubject.objects.get(
            registration_identifier=obj.to_string(obj.registration_identifier)
        )
        self.assertEqual(rs.site.pk, 10)

    @override_settings(SITE_ID=SiteID(20))
    def test_site2(self):
        add_or_update_django_sites(single_sites=self.default_sites, verbose=False)
        obj = SubjectModelOne.objects.create(screening_identifier="12345")
        rs = RegisteredSubject.objects.get(
            registration_identifier=obj.to_string(obj.registration_identifier)
        )
        self.assertEqual(rs.site.pk, 20)



# Generated by Django 4.2.1 on 2023-07-05 02:16

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("edc_registration", "0026_historicalregisteredsubject_familiar_name_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="registeredsubject",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ["subject_identifier"],
                "permissions": (
                    ("display_firstname", "Can display first name"),
                    ("display_lastname", "Can display last name"),
                    ("display_dob", "Can display DOB"),
                    ("display_identity", "Can display identity number"),
                    ("display_initials", "Can display initials"),
                ),
                "verbose_name": "Registered Subject",
                "verbose_name_plural": "Registered Subjects",
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-08-10 10:32
from __future__ import unicode_literals

import _socket
import edc_model_fields.fields.userfield
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("edc_registration", "0007_auto_20170321_1119")]

    operations = [
        migrations.AddField(
            model_name="registeredsubject",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="registeredsubject",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="registeredsubject",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
    ]



import sys

from django.apps import AppConfig as DjangoAppConfig
from django.apps import apps as django_apps


class AppConfig(DjangoAppConfig):
    name = "edc_registration"
    verbose_name = "Edc Registration"
    app_label = "edc_registration"
    has_exportable_data = True
    include_in_administration_section = True

    def ready(self):
        sys.stdout.write(f"Loading {self.verbose_name} ...\n")
        sys.stdout.write(f"  * using {self.app_label}.registeredsubject\n")
        sys.stdout.write(f" Done loading {self.verbose_name}.\n")

    @property
    def model(self):
        return django_apps.get_model(self.app_label, "registeredsubject")



# Generated by Django 4.2.1 on 2023-07-07 19:32

import edc_sites.models
from django.db import migrations

import edc_registration.models.managers


class Migration(migrations.Migration):
    dependencies = [
        ("edc_registration", "0027_alter_registeredsubject_options"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="registeredsubject",
            managers=[
                (
                    "objects",
                    edc_registration.models.managers.RegisteredSubjectManager(),
                ),
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
    ]
